



#include <stdlib.h>
#include "nes.h"
#include "sdl_saving.h"

extern "C" {
#include "rc.h"
#include "debug.h"

    int rc_bindkey(char *keyname, char *cmd);
    int rc_unbindkey(char *keyname);
    void rc_unbindall(void);
    int rc_sourcefile(char *filename);
    int splitline(char **argv, int max, char *line);
    int rc_command(char *line);
}

extern NES *g_emu;
extern bool exit_game_loop;
extern bool interrupt_game_loop;

/*
 * define the command functions for the controller pad.
 */

#define CMD_PAD(b, B) \
static int (cmd_ ## b)(int c, char **v) \
{ \
    NES_pad *pad = g_emu->get_pad(0); \
    pad->nes_ ## B = v[0][0] == '+'; \
    return 0; \
}

// static int (cmd_ ## b)(int c, char **v)

CMD_PAD(up, UP);
CMD_PAD(down, DOWN);
CMD_PAD(left, LEFT);
CMD_PAD(right, RIGHT);
CMD_PAD(a, A);
CMD_PAD(b, B);
CMD_PAD(start, START);
CMD_PAD(select, SELECT);

/*
 * the set command is used to set rc-exported variables.
 */

static int cmd_set(int argc, char **argv)
{
	if (argc < 3)
		return -1;
	return rc_setvar(argv[1], argc-2, argv+2);
}



/*
 * the following commands allow keys to be bound to perform rc commands.
 */

static int cmd_bind(int argc, char **argv)
{
	if (argc < 3)
		return -1;
	return rc_bindkey(argv[1], argv[2]);
}

static int cmd_unbind(int argc, char **argv)
{
	if (argc < 2)
		return -1;
	return rc_unbindkey(argv[1]);
}

static int cmd_unbindall(int argc, char **argv)
{
	rc_unbindall();
	return 0;
}

static int cmd_source(int argc, char **argv)
{
	if (argc < 2)
		return -1;
	return rc_sourcefile(argv[1]);
}

static int cmd_quit(int argc, char **argv)
{
    exit_game_loop = true;
    interrupt_game_loop = true;
// 	exit(0);
	/* NOT REACHED */
}

static int cmd_reset(int argc, char **argv)
{
    g_emu->reset();
	return 0;
}

static int cmd_savestate(int argc, char **argv)
{
    if (snss_save(g_emu)) {
        LOG("snapsave");
    }
#if 0
    char filename[512];

    snprintf(filename, sizeof(filename),
             "%s/%s.%d",
             rc_getstr("snapdir"),
             g_emu->getROMname(),
             rc_getint("saveslot"));
    if (g_emu->saveState(filename))
    {
        LOG("snapsave to");
        LOG(filename);
    }
	return 0;
#endif
}

static int cmd_loadstate(int argc, char **argv)
{
    if (snss_load(g_emu)) {
        LOG("snapload");
    }
#if 0
    char filename[512];

    snprintf(filename, sizeof(filename),
             "%s/%s.%d",
             rc_getstr("snapdir"),
             g_emu->getROMname(),
             rc_getint("saveslot"));
    if (g_emu->loadState(filename))
    {
        LOG("snapload from");
        LOG(filename);
    }
#endif
	return 0;
}

static int cmd_diskside(int argc, char **argv)
{
    int side = (argc > 1) ? atoi(argv[1]) - 1 : 0;
    g_emu->SetDiskSide((side & 0x03) + 1);
    fprintf(stderr, "SetDiskSide(%d)\n", (side & 0x03) + 1);
    return 0;
}

static int cmd_incsaveslot(int argc, char **argv)
{
    int slot = rc_getint("saveslot");
    char buf[64];
    slot = (++slot) & 0x07;
    snprintf(buf, sizeof(buf), "set saveslot %d", slot);
    return rc_command(buf);
}

static int cmd_decsaveslot(int argc, char **argv)
{
    int slot = rc_getint("saveslot");
    char buf[64];
    slot = (--slot) & 0x07;
    snprintf(buf, sizeof(buf), "set saveslot %d", slot);
    return rc_command(buf);
}

/*
 * table of command names and the corresponding functions to be called
 */

rccmd_t rccmds[] =
{
	RCC("set", cmd_set),
	RCC("bind", cmd_bind),
	RCC("unbind", cmd_unbind),
	RCC("unbindall", cmd_unbindall),
	RCC("source", cmd_source),
	RCC("reset", cmd_reset),
	RCC("quit", cmd_quit),
	RCC("savestate", cmd_savestate),
	RCC("loadstate", cmd_loadstate),
    RCC("diskside", cmd_diskside),
    RCC("incsaveslot", cmd_incsaveslot),
    RCC("decsaveslot", cmd_decsaveslot),

	RCC("+up", cmd_up),
	RCC("-up", cmd_up),
	RCC("+down", cmd_down),
	RCC("-down", cmd_down),
	RCC("+left", cmd_left),
	RCC("-left", cmd_left),
	RCC("+right", cmd_right),
	RCC("-right", cmd_right),
	RCC("+a", cmd_a),
	RCC("-a", cmd_a),
	RCC("+b", cmd_b),
	RCC("-b", cmd_b),
	RCC("+start", cmd_start),
	RCC("-start", cmd_start),
	RCC("+select", cmd_select),
	RCC("-select", cmd_select),

	RCC_END
};




extern "C"
int rc_command(char *line)
{
	int i, argc, ret;
	char *argv[128], *linecopy;

	linecopy = (char *)malloc(strlen(line)+1);
	strcpy(linecopy, line);

	argc = splitline(argv, (sizeof argv)/(sizeof argv[0]), linecopy);
	if (!argc)
	{
		free(linecopy);
		return -1;
	}

	for (i = 0; rccmds[i].name; i++)
	{
		if (!strcmp(argv[0], rccmds[i].name))
		{
			ret = rccmds[i].func(argc, argv);
			free(linecopy);
			return ret;
		}
	}

	printf("unknown command: %s\n", argv[0]);
	free(linecopy);

	return -1;
}

